<?php

namespace App\Http\Controllers;

use DB;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Route;
use Spatie\Permission\Models\Permission;
use Yajra\Datatables\Datatables;
use Auth;
use App\Models\User;

class RolesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    function __construct()
    {
        $this->middleware(function ($request, $next) { 
         $user = User::find(Auth::User()->id);
        if($user->hasPermissionTo('roles'))
        {
            return $next($request);
        }else
        {
            return abort(401);
        }
        //return $next($request);
       });
    }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {   
        /*$roles = Role::orderBy('id','DESC')->paginate(5);
        return view('roles.index',compact('roles'))
            ->with('i', ($request->input('page', 1) - 1) * 5);*/
            return view('admin.roles.index',[]);
            
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $permissions = Permission::whereNotIn("user_friendly_name",['permissions'])->groupBy("user_friendly_name")->get();
        return view('admin.roles.create', compact('permissions'));
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => 'required|unique:roles,name',
            'permission' => 'required',
        ]);
    
    //dd($request->get('permission'));
        $role = Role::create(['name' => $request->get('name')]);
        $role->syncPermissions($request->get('permission'));
        /*foreach($request->get('permission') as $permission)
        {
            $getPerm = Permission::where("user_friendly_name",$permission)->get();
            foreach($getPerm as $prm)
            {
                $perm = DB::table("role_has_permissions")->insert([
                                'permission_id' => $prm->id,
                                'role_id' => $role->id
                            ]);
                
            }
            
            
        }*/
    
        return redirect()->route('roles.index')
                        ->with('success','Role created successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    /*public function show(Role $role)
    {
        $role = $role;
        //dd($role->groupBy("user_friendly_name")->pluck());
        $rolePermissions = $role->permissions;
    
        return view('roles.show', compact('role', 'rolePermissions'));
    }*/
    public function show(Role $role)
    {
        $role = $role;
        //dd($role->permissionsFriendlyName);
        $rolePermissions = $role->permissionsFriendlyName;
    
        return view('admiin.roles.show', compact('role', 'rolePermissions'));
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Role $role)
    {
        //$role = $role;
       // dd($role);
        //$rolePermissions = $role->permissions->pluck('name')->toArray();
        $rolePermissions = $role->permissionsFriendlyName->pluck('user_friendly_name')->toArray();
        $permissions = Permission::whereNotIn("user_friendly_name",['permissions'])->groupBy("user_friendly_name")->get();
    
        return view('admin.roles.edit', compact('role', 'rolePermissions', 'permissions'));
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Role $role, Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'permission' => 'required',
        ]);
        
        $role->update($request->only('name'));
    
        $role->syncPermissions($request->get('permission'));
    
        return redirect()->route('roles.index')
                        ->with('success','Role updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Role $role)
    {
        $role->delete();

        return redirect()->route('roles.index')
                        ->with('success','Role deleted successfully');
    }
    
    /**
     * Delete role data
     * 
     * @param Request $request
     * 
     * @return \Illuminate\Http\Response
     */
     public function delRole(Request $request) 
    {
        if($request->id!='')
        {
            $us = Role::where("id",$request->id)->delete();
            if($us)
            {
                DB::table("role_has_permissions")->where("role_id",$request->id)->delete();
                echo "1";
            }else
            {
                echo "0";
            }
            
        }
        return "";
    }
    
    public function tableList()
    {
        //dd(User::with('role')->get()->toArray());
        //$users = User::where("user_type",'web');
        $roles = Role::where("name",'!=','Super Admin');
       // $users = User::join("model_has_roles","model_has_roles.modal_id",'=',"users.id")->where("users.user_type",'web');
            return Datatables::of($roles)    
            ->addColumn('action', function ($user) {

                 //$hidetake = "display:none;";
                 $hidetake = "";
                 $edit = route('roles.edit', $user->id);
                $chk = ($user->status=='1') ? "checked":"";
                $b = '<a  href="'.$edit.'" class="btn btn-sm btn-primary"><i class="fa fa-edit"></i></a> ';
                $b.= '<a onclick="del_client('.$user->id.')" href="javascript:;" class="btn btn-sm btn-danger danger-alert"><i class="fa fa-trash"></i></a> ';                                
                
                
                
                //$b.='<div class="my_switch make-switch"> <input type="checkbox" name="user_st" value="'.$user->id.'" checked /></div>';
                return $b;
            })
            
            ->editColumn('created_at',function($camp) {
                
              return $camp->created_at->format("Y-m-d h:i a");
            })
            ->rawColumns(['action'])
           // ->removeColumn('status')
            ->make(true);
    }
}
