<?php

use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

Route::get('/', function () {
    return view('welcome');
});

Route::get('/dashboard', function () {
    return view('admin.dashboard');
})->middleware(['auth'])->name('dashboard');

Route::get('stripe-flow',"App\Http\Controllers\StripeController@StripeFlow");
require __DIR__.'/auth.php';

Route::group(['namespace' => 'App\Http\Controllers'], function(){

    //Route::middleware(['auth'],function(){
    Route::group(['middleware' => 'auth'],function(){
        
        Route::get('login-client',"UsersController@CustomLogin")->name('login-client');
        
        Route::get('/home', 'HomeController@index')->name('home.index');
        //Route::get('/', 'HomeController@index')->name('home.index');
        Route::get('/profile', 'HomeController@profile');
        Route::post('/save-profile', 'HomeController@saveProfile')->name('saveProfile');
    
        Route::get('/logout', function(){})->name('logout.perform');
        
        Route::group(['prefix' => 'users'], function() {
                    Route::get('/', 'UsersController@index')->name('users.index');
                    Route::get('/create', 'UsersController@create')->name('users.create');
                    Route::post('/create', 'UsersController@store')->name('users.store');
                    Route::get('/{user}/show', 'UsersController@show')->name('users.show');
                    Route::get('/{user}/edit', 'UsersController@edit')->name('users.edit');
                    Route::patch('/{user}/update', 'UsersController@update')->name('users.update');
                    Route::delete('/{user}/delete', 'UsersController@destroy')->name('users.destroy');
                    Route::get("/users-list",'UsersController@tableList')->name('getUsers');
                    Route::get("/del-user",'UsersController@DelUser')->name('delUser');
                    
                    
                });
                
        Route::group(['prefix' => 'posts'], function() {
                    Route::get('/', 'PostsController@index')->name('posts.index');
                    Route::get('/create', 'PostsController@create')->name('posts.create');
                    Route::post('/create', 'PostsController@store')->name('posts.store');
                    Route::get('/{post}/show', 'PostsController@show')->name('posts.show');
                    Route::get('/{post}/edit', 'PostsController@edit')->name('posts.edit');
                    Route::patch('/{post}/update', 'PostsController@update')->name('posts.update');
                    Route::delete('/{post}/delete', 'PostsController@destroy')->name('posts.destroy');
                });
                
         //system roles       
        Route::resource('roles', RolesController::class);
        Route::get('del-role','RolesController@delRole')->name("delRole");
        Route::get('roles-list','RolesController@tableList')->name("getRoles");
        
        //system permissions
        Route::resource('permissions', PermissionsController::class);
        
        
        
        //Settings
        Route::resource('setting', SettingController::class);
        Route::post('/save-settings', 'SettingController@saveSettings')->name('save_settings');
        Route::get("new-reminder","SettingController@addReminder")->name("new_reminder");
        Route::get("del-reminder","SettingController@delReminder")->name('del_reminder');
        Route::get('/userdata/twilionums', 'SettingController@getNums')->name('twilionums');
        
        //Extra Services
        Route::resource('extra-services', ExtraServiceController::class);
        Route::get('extra-services-list','ExtraServiceController@tableList')->name("getExtraServices");
        Route::get('del-ex-serice','ExtraServiceController@delSerivce')->name("delService");
        
        //Service type cats
        Route::resource('service-type-cats', ServiceTypeCatsController::class);
        Route::get('cat-services-list','ServiceTypeCatsController@tableList')->name("getcatServices");
        Route::get('del-cat-service','ServiceTypeCatsController@delSerivce')->name("delCatService");
        Route::get('del-cat-service-price','ServiceTypeCatsController@delSerivceprice')->name("delcatServicePrice");
        
        //Services type
        Route::resource('service-type', ServiceTypeController::class);
        Route::get('services-type-list','ServiceTypeController@tableList')->name("getServicestype");
        Route::get('del-cat-serice','ServiceTypeController@delSerivce')->name("delServiceType");
        
        //Booking
        Route::resource('booking', BookingController::class);
        Route::get('get-book-cat-service','BookingController@getBookCatServices')->name("getBookCatServices");
        Route::get('get-book-ext-service','BookingController@getBookextServices')->name("getBookextServices");
        
        //Customer
        Route::resource('customers', CustomerController::class);
        Route::get("/customer-list",'CustomerController@tableList')->name('getCustomers');
        Route::get("/billing-info",'CustomerController@billingInfo')->name('billingInfo');
        
        Route::get("/cards-list",'CustomerController@CardsList')->name('cardList');
        Route::post("/save-card",'CustomerController@saveCard')->name('saveCard');
        
        Route::get("/addresses",'CustomerController@addressIndex');
        Route::get("/address-list",'CustomerController@AddressList')->name('AddressList');
        Route::post("/save-address",'CustomerController@saveAddress')->name('saveAddress');
        Route::get("/address-del",'CustomerController@delAddress')->name('delAddress');
        Route::get("/address-get",'CustomerController@getAddress')->name('getAddress');
        
    });
        

});

